import React, { useRef, useEffect, useState, useContext } from 'react'
import { ConfigProvider, message, Modal } from 'antd'
import Generator, { defaultCommonSettings, defaultGlobalSettings, defaultSettings } from 'fr-generator'
import { settings, baseSettings, globalSettings } from './config'
import widgets from '../widgets'
import { saveTableConfig, GetTableConfigJson } from '../../apis/process'
import FormRender from '../FormRender'

const FormDesigner = (props) => {

  const { tableName } = props

  const { getPrefixCls } = useContext(ConfigProvider.ConfigContext)
  const prefixCls = getPrefixCls('pandaXform')
  const [visible, setVisible] = useState(false)
  const [schema, setSchema] = useState({})
  const ref = useRef(null)

  const getTableField = () => {
    console.log('setSchema', settings)
  }

  const getTableConfig = async () => {
    const { code, data } = await GetTableConfigJson({ tableName })
    if (code === 0) {
      if (typeof data === 'string') {
        setTimeout(() => {
          ref && ref.current && ref.current.setValue && ref.current.setValue(JSON.parse(data))
          setSchema(JSON.parse(data))
        }, 0)
      }
    } else {
      message.error('获取表单配置失败')
    }
  }

  useEffect(() => {
    getTableConfig()
  }, [])

  const extraButtons = [
    false,
    true,
    true,
    true,
    {
      text: '预览',
      onClick: (e) => {
        setVisible(true)
        setSchema(ref.current.getValue())
      }
    },
    {
      text: '提交',
      onClick: async (value) => {
        const errors = ref.current.getErrorFields()
        if (errors.length) {
          return message.error('请按照提示完善表单内容')
        }
        const { code, data, msg } = await saveTableConfig(ref.current.getValue())
        if (code === 0) {
          message.info('保存成功')
        } else {
          message.error(msg)
        }
      }
    }
  ]

  const onChange = (value) => {
    // console.log('value', value)
  }

  const onCanvasSelect = (schema) => {
    // console.log('schema', schema)
  }

  return (
    <div className={prefixCls} style={{ height: '750px' }}>
      <Generator
        configProvider={{ prefixCls: prefixCls }}
        mapping={{
          object: 'Header',
        }}
        validation={true}
        ref={ref}
        extraButtons={extraButtons}
        onChange={onChange}
        widgets={widgets}
        onCanvasSelect={onCanvasSelect}
        settings={settings}
        commonSettings={baseSettings}
        globalSettings={globalSettings}
      />
      <Modal
        title={tableName}
        width='80%'
        bodyStyle={{ height: '700px', overflow: 'auto' }}
        onOk={() => setVisible(false)}
        onCancel={() => setVisible(false)}
        visible={visible}
      >
        <FormRender schema={schema} />
      </Modal>
    </div>

  )

}

export default FormDesigner