import React, { useEffect, useMemo, useState, useRef, useContext } from 'react'
import styles from './index.less'
import {
  GetAccountPageList,
  GetAccountConfigInfo,
  GetTableGroupMeta,
  GetTableDataInfo,
  SaveTableDataInfo,
  EditTableDataInfo,
  DeleteTableDataInfo
} from '../../../../apis/process'
import { Table, Button, Modal, message, Popconfirm } from 'antd'
import { SnippetsOutlined, PlusOutlined, FormOutlined, DeleteOutlined, } from '@ant-design/icons'
import { GlobalStore } from '../../../FormRender'

const icons = {
  '详情': <SnippetsOutlined />,
  '编辑': <FormOutlined />,
  '删除': <DeleteOutlined />,
}
let formStateStr = '添加'

const RelationForm = ({ value, onChange, schema }) => {

  const fieldNames = ['事件编号', '工单编号']
  const { disabled, source } = schema
  const configInfo = source ? { ...JSON.parse(source) } : {}

  // const { listData, setListData, parent } = useContext(GlobalStore)

  // console.log(parseFormData, setParseFormData)

  const [listData, setListData] = useState([])

  const { codes } = parent

  const [config, setConfig] = useState({ webShowFieldGroup: '', fieldGroup: '' })
  const [dataSource, setDataSource] = useState([])
  const [groupMeta, setGroupMeta] = useState([])
  const [formData, setFormData] = useState([])
  const [visible, setVisible] = useState(false)
  const [formState, setFormState] = useState('详情')
  const [row, setRow] = useState({})
  const parseFormRef = useRef(null)

  // const getMsg = () => {
  //   let fields = configInfo['映射字段'].map(v => v.toField)
  //   return `台账【${configInfo['台账名称']}】缺少${fields.join('、')}字段`
  // }

  // const show = useMemo(() => {
  //   let str = configInfo['控制规则']
  //   try {
  //     let matchArr = str ? str.match(/\{(.+?)\}/g) : []
  //     matchArr?.forEach(v => {
  //       let value = v.replace(/{/g, '').replace(/}/g, '')
  //       let valueArr = value ? value.split('.') : []
  //       let fieldValue = parseFormData[valueArr[1]]
  //       str = str.replace(v, `'${fieldValue}'`)
  //     })
  //     str = str.replace(/=/g, '==')
  //     str = str.replace(/and/g, '&&')
  //     str = str.replace(/or/, '||')
  //     if (str && eval(str)) {
  //       return !configInfo['默认显示']
  //     }
  //     return configInfo['默认显示']
  //   } catch {
  //     message.error('字符串规则解析失败')
  //     return configInfo['默认显示']
  //   }
  // }, [parseFormData, configInfo])

  // const localForm = useMemo(() => {
  //   const { addFieldGroup } = config
  //   let array = []
  //   let field = addFieldGroup ? addFieldGroup.split(',') : []
  //   const getFieldValue = (s, parseFormData, codes) => {
  //     return codes ? (codes[s.fromField] || parseFormData[s.fromField]) : parseFormData[s.fromField]
  //   }
  //   field.forEach(v => {
  //     configInfo['映射字段'].forEach(s => {
  //       if (s.toField === v) {
  //         array.push({ fieldName: s.toField, fieldValue: getFieldValue(s, parseFormData, codes) })
  //       }
  //     })
  //   })
  //   return array
  // }, [parseFormData, config, configInfo, codes, dataSource])

  const localForm = []

  const tablesSchema = useMemo(() => {
    const { fieldGroup, editFieldGroup, addFieldGroup } = config
    let field = []
    if (formState === '详情') {
      field = fieldGroup ? fieldGroup.split(',') : []
    } else if (formState === '添加') {
      field = addFieldGroup ? addFieldGroup.split(',') : []
    } else if (formState === '编辑') {
      field = editFieldGroup ? editFieldGroup.split(',') : []
    }
    let groups = []
    groupMeta.forEach(v => {
      let schema = []
      v.schema.forEach(s => {
        if (field.includes(s.fieldName)) {
          schema.push({ ...s, readOnly: fieldNames.includes(s.fieldName) ? 1 : s.readOnly })
        }
      })
      if (schema.length > 0) {
        groups.push({ ...v, schema, })
      }
    })
    return {
      groups,
      values: formState === '添加' ? localForm : formData
    }
  }, [formState, localForm, groupMeta, formData, config])

  const groupMetaInfo = useMemo(() => {
    let array = []
    const { webShowFieldGroup } = config
    let field = webShowFieldGroup ? webShowFieldGroup.split(',') : []
    let groupArr = groupMeta.length ? groupMeta.reduce((f, n) => f.concat(n.schema), []) : []
    if (field.length > 0) {
      field.forEach(v => {
        groupArr.forEach(s => {
          if (s.fieldName === v) {
            array.push(s)
          }
        })
      })
    }
    return array
  }, [groupMeta, config])

  const columns = useMemo(() => {
    let btn = ['详情']
    if (!disabled) {
      btn = ['详情', '编辑', '删除']
    }
    let array = []
    if (groupMetaInfo.length > 0) {
      array.push({ title: '序号', width: 50, align: 'center', render: (_, r, i) => i + 1 })
      groupMetaInfo.forEach(v => {
        array.push({
          title: v.alias,
          dataIndex: v.fieldName,
          key: v.fieldName,
          ellipsis: {
            visibleTitle: false,
          },
          // width: 120
        })
      })
      array.push({
        title: '操作',
        dataIndex: '',
        key: 'action',
        width: btn.length === 1 ? 80 : btn.length * 40,
        fixed: 'right',
        render: (t, r) => {
          return (
            <div style={{ display: 'flex', justifyContent: 'center' }}>
              {
                btn.map(v => {
                  if (v === '删除') {
                    return (
                      <Popconfirm
                        // disabled={selectedRowKeys.length}
                        title='确定要删除数据吗？'
                        onConfirm={() => rowClick(v, r)}
                        onCancel={() => { }}
                        okText='确定'
                        cancelText='取消'
                      >
                        <Button type='link' icon={icons[v]} title={v} />
                      </Popconfirm>
                    )
                  }
                  return (
                    <Button
                      type='link'
                      icon={icons[v]}
                      title={v}
                      onClick={() => rowClick(v, r)}
                    />
                  )
                })
              }
            </div>
          )
        }
      })
    }
    return array
  }, [disabled, groupMetaInfo, dataSource, formState])

  const isReadOnly = useMemo(() => {
    if (formState === '编辑' || formState == '添加') {
      return false
    }
    return true
  }, [formState])

  const rowClick = async (v, r) => {
    setRow(r)
    if (v === '详情') {
      setFormData(getItem(r))
      setVisible(true)
    } else if (v === '编辑') {
      setFormData(getItem(r))
      setVisible(true)
    } else if (v === '删除') {
      deleteTableInfo(r)
    }
    formStateStr = v
    setFormState(v)
  }

  const getItem = (data, max) => {
    if (Array.isArray(data)) {
      let obj = { ...row, ID: row.ID }
      data.forEach(v => {
        obj[v.fieldName] = v.fieldValue
      })
      return obj
    } else {
      let values = []
      for (let key in data) {
        if (max) {
          values.push({ FieldName: key, FieldValue: data[key] })
        } else {
          values.push({ fieldName: key, fieldValue: data[key] })
        }
      }
      return values
    }
  }

  const deleteTableInfo = (r) => {
    let listItem = { type: 'delete', accountTable: configInfo['台账名称'], id: r.ID, values: getItem('delete', r) }
    if (r.ID.includes('前端ID')) {
      let list = listData.filter(v => v.id !== r.ID)
      setListData(list)
    } else {
      setListData([...listData, listItem])
    }
    let tableData = dataSource.filter(v => v.ID !== r.ID)
    setDataSource(tableData)
    onChange(`${tableData.length}`)
  }

  const getTableInfo = async (r) => {
    const { code, data } = await GetTableDataInfo({ accountName: configInfo['台账名称'], id: Number(r.ID) })
    if (code === 0) {
      setFormData(data)
      setVisible(true)
    } else {
      message.error('数据请求失败！')
    }
  }

  const getConfig = async () => {
    let { code, data, msg } = await GetAccountConfigInfo(configInfo['台账名称'])
    if (code === 0) {
      setConfig(data)
      getTableData(data.addFieldGroup ? data.addFieldGroup.split(',') : [])
    } else {
      message.error(msg)
    }
  }

  const getGroupMeta = async () => {
    const { code, data, msg } = await GetTableGroupMeta(configInfo['台账名称'])
    if (code === 0) {
      setGroupMeta(data)
    } else {
      message.error(msg)
    }
  }

  const getQueryWheres = (addFieldGroup) => {
    let queryWheres = []
    addFieldGroup.forEach(v => {
      configInfo['映射字段'].forEach(s => {
        if (s.toField === v) {
          queryWheres.push({ field: s.toField, type: '等于', value: codes ? (codes[s.fromField] || parseFormData[s.fromField]) : parseFormData[s.fromField] })
        }
      })
    })
    return queryWheres
  }

  const getTableData = async (addFieldGroup) => {
    // let queryWheres = getQueryWheres(addFieldGroup)
    // if (!queryWheres.length) {
    //   onChange(`出错了,${getMsg()}`)
    // }
    let params = {
      accountName: configInfo['台账名称'],
      direction: 'desc',
      timeField: '录入时间',
      pageIndex: 0,
      pageSize: 0,
      // queryWheres
    }
    // if (queryWheres.every(v => !v.value)) return
    const { code, data, msg } = await GetAccountPageList(params)
    if (code === 0) {
      let tableData = JSON.parse(data.jsonData)
      if (Array.isArray(tableData)) {
        setDataSource(tableData)
        onChange(`${tableData.length}`)
      } else {
        setDataSource([])
        onChange(`${0}`)
      }
    } else {
      message.error(msg)
    }
  }

  const getData = () => {
    getConfig()
    getGroupMeta()
  }

  const onOk = async () => {
    let formData = parseFormRef.current.getFormValues()
    let type = row.ID.includes('前端ID') ? 'add' : 'edit'
    let itemData = { type, accountTable: configInfo['台账名称'], id: row.ID, values: formData }
    if (formStateStr !== '添加') {
      let array = []
      dataSource.forEach(v => {
        if (v.ID === row.ID) {
          array.push({ ...v, ...getItem(formData) })
        } else {
          array.push({ ...v })
        }
      })
      setDataSource(array)
      onChange(`${array.length}`)
    } else if (formStateStr === '添加') {
      let array = [getItem(formData), ...dataSource]
      setDataSource(array)
      onChange(`${array.length}`)
    }
    let listArray = listData.filter(v => v.id !== itemData.id)
    listArray.push({ ...itemData })
    setListData(listArray)
    setVisible(false)
  }

  const addClick = () => {
    // if (!localForm.length) {
    //   return message.error(getMsg())
    // }
    setRow({ ID: `前端ID${new Date().getTime()}` })
    setFormState('添加')
    formStateStr = '添加'
    setVisible(true)
  }

  useEffect(() => {
    console.log('configInfo', configInfo)

    if (configInfo['台账名称']) {
      getData()
    }
  }, [configInfo['台账名称']])

  return (
    <div className={styles.relationForm}>
      <div className={styles.header}>
        {disabled ? null : <Button className={styles.headerBtn} type='primary' size='middle' icon={<PlusOutlined />} onClick={addClick}>添加</Button>}
      </div>
      <Table
        rowKey={r => JSON.stringify(r)}
        size='small'
        bordered
        columns={columns}
        dataSource={dataSource}
        pagination={{ total: dataSource.length, showTotal: (value) => `总计 ${value} 条` }}
      />
      <Modal
        title={schema.name}
        width='90%'
        visible={visible}
        onOk={onOk}
        onCancel={() => setVisible(false)}
        bodyStyle={{ height: 700 }}
        destroyOnClose
      >
        {/* <JsonToHtml
          isReadOnly={isReadOnly}
          tablesSchema={tablesSchema}
          ref={parseFormRef}
          noButtons
        /> */}
      </Modal>
    </div>
  )

}

export default RelationForm