import React, { useEffect, useMemo } from 'react'
import { Cascader } from 'antd'
import city from '../../../../constant/city'

const CascadeSelector = (props) => {

  const { value, onChange, schema } = props
  const { presetValue, disabled, soruceType, options, dictionary } = schema

  const enums = useMemo(() => {
    if (soruceType === '城市数据') {
      return city
    }
    return []
  }, [soruceType])

  const getDefaultValue = selectedValues => {
    if (!selectedValues || !selectedValues.length) {
      return []
    }

    const result = [];

    // 递归查询城市名称数据
    function findLabel(children) {
      children.forEach(item => {
        const { label, value, children } = item;
        if (selectedValues.includes(label)) {
          result.push(value);
        }
        if (children && children.length) {
          findLabel(children);
        }
      });
    }

    // 卡省份解决县或市名称一样的问题
    city.forEach(item => {
      const { label, value, children } = item;
      if (selectedValues.includes(label)) {
        result.push(value);
        if (children && children.length) {
          findLabel(children);
        }
      }
    });
    // 设置默认值
    return result;
  }

  const cascaderChange = selectedValues => {
    const result = []
    // 递归查询城市所有数据
    function findCityData(city) {
      city.forEach(item => {
        const { id, label, value, children } = item;
        if (selectedValues.includes(value)) {
          result.push({ id, label, value })
          if (children && children.length) {
            findCityData(children)
          }
        }
      })
    }
    findCityData(city)
    // 分发监听
    const cityValue = result.map(item => {
      return item.label;
    })
    onChange && onChange(cityValue.join('/'));
  }

  useEffect(() => {
    onChange(presetValue)
  }, [presetValue])

  return (
    <Cascader
      value={value ? getDefaultValue(value) : []}
      onChange={cascaderChange}
      disabled={disabled}
      style={{ width: '100%' }}
      options={enums}
    />
  )

}

export default CascadeSelector