import React, { useEffect, useState } from 'react'
import { Form, Space, TreeSelect, Button, Input, Modal, message } from 'antd'
import { MinusCircleOutlined, PlusOutlined } from '@ant-design/icons'
import styles from './index.less'
import { ReloadTableFields } from '../../../../../apis/process'

const { TreeNode } = TreeSelect

const Fieldshine = (props) => {

  console.log('props', props)

  const { addons, onChange, value } = props
  const { accountName } = addons.formData

  const [form] = Form.useForm()
  const [visible, setVisible] = useState(false)
  const [fromField, setFromField] = useState([])
  const [toField, setToField] = useState([])

  const onOk = () => {
    const { field } = form.getFieldsValue()
    onChange(field)
    setVisible(false)
  }

  const inputClick = async () => {
    if (!accountName) {
      return message.info('请选择台账名称')
    }
    const req1 = ReloadTableFields({ tableName: '设备_台账_户表' })
    const req2 = ReloadTableFields({ tableName: '事件_测试02' })
    const [res1, res2] = await Promise.all([req1, req2])
    if (res1.code === 0) {
      if (Array.isArray(res1.data.root)) {
        setFromField(res1.data.root)
      }
    } else {
      message.error(res1.msg)
    }
    if (res2.code === 0) {
      if (Array.isArray(res2.data.root)) {
        setToField(res2.data.root)
      }
    } else {
      message.error(res2.msg)
    }
    setVisible(true)
  }

  // useEffect(() => {
  //   addons.setValue(addons.dataPath, [])
  // }, [accountName])

  return (
    <div className={styles.fieldshine}>
      <Input
        value={value && value.length ? JSON.stringify(value) : null}
        onClick={inputClick}
      />
      <Modal
        getContainer={false}
        width='500px'
        title='字段映射'
        visible={visible}
        onCancel={() => setVisible(false)}
        onOk={onOk}
      >
        <Form name="dynamic_form_nest_item" form={form} autoComplete="off">
          <Form.List name="field">
            {(value, { add, remove }) => (
              <>
                {value.map(({ key, name, ...restField }) => (
                  <Space
                    key={key}
                    style={{
                      display: 'flex',
                      marginBottom: '5px                                           ',
                      justifyContent: 'center',
                    }}
                    align="baseline"
                  >
                    <Form.Item
                      {...restField}
                      name={[name, 'fromField']}
                      rules={[
                        {
                          required: true,
                          message: 'Missing first name',
                        },
                      ]}
                    >
                      <TreeSelect
                        showSearch
                        style={{ width: '180px' }}
                        dropdownStyle={{ maxHeight: 400, overflow: 'auto' }}
                        placeholder="请选择字段名 "
                        allowClear
                        treeDefaultExpandAll
                        showCheckedStrategy
                      >
                        <TreeNode value='设备_台账_户表' title='设备_台账_户表' disabled>
                          {fromField.map(s => <TreeNode value={s.alias} title={s.name} key={s.name} />)}
                        </TreeNode>
                      </TreeSelect>
                    </Form.Item>
                    <div className={styles.centerImg}></div>
                    <Form.Item
                      {...restField}
                      name={[name, 'toField']}
                      rules={[
                        {
                          required: true,
                          message: 'Missing last name',
                        },
                      ]}
                    >
                      <TreeSelect
                        showSearch
                        style={{ width: '180px' }}
                        dropdownStyle={{ maxHeight: 400, overflow: 'auto' }}
                        placeholder="请选择字段名 "
                        allowClear
                        treeDefaultExpandAll
                        showCheckedStrategy
                      >
                        <TreeNode value='事件_测试02' title='事件_测试02' disabled>
                          {toField.map(s => <TreeNode value={s.alias} title={s.name} key={s.name} />)}
                        </TreeNode>
                      </TreeSelect>
                    </Form.Item>
                    <MinusCircleOutlined onClick={() => remove(name)} />
                  </Space>
                ))}
                <Form.Item style={{ padding: '0 10px' }}>
                  <Button type="dashed" onClick={() => add()} block icon={<PlusOutlined />}>
                    添加映射
                  </Button>
                </Form.Item>
              </>
            )}
          </Form.List>
        </Form>
      </Modal>

    </div>
  )

}

export default Fieldshine